@@ -1,6 +1,6 @@
 # [[[cog import cog; cog.outl('"""\n%s\n"""' % file('../README.rst').read()) ]]]
 """
-NEOrg - Numerical Simulation Organizer
+NEOrg - Numerical Experiment Organizer
 ======================================
 
 NEOrg is a wiki to organize your experimental data.
@@ -51,7 +51,7 @@ Quick start
 Dependencies
 ------------
 
-- `docutils`, `Flask` and `argparse`
+- `docutils`, `Flask`, `Whoosh` and `argparse`
 - `PyYAML` to load YAML data (optional).
 
 For development:
@@ -67,16 +67,21 @@ Links
 - `project page`_
 - document_
 - `issue tracking`_
+- `change log`_
+- You can see screenshots in
+  `my blog post <http://tkf.github.com/2011/06/04/neorg-0.0.1.html>`_.
 
 .. _`project page`: https://bitbucket.org/tkf/neorg/
 .. _document: http://tkf.bitbucket.org/neorg-doc/index.html
 .. _`issue tracking`: https://bitbucket.org/tkf/neorg/issues
+.. _`change log`: http://tkf.bitbucket.org/neorg-doc/changelog.html
+
 
 """
 # [[[end]]]
 
 __author__  = "Takafumi Arakaki"
-__version__ = "0.0.2"
+__version__ = "0.0.3"
 __license__ = "MIT License"
 
 
