@@ -1,10 +1,11 @@
 def serve(port, root=None, debug=None, browser=None):
-    from neorg.web import app, update_system_info
+    from neorg.web import app, update_system_info, update_search_index
     from neorg.config import load_config
     from neorg.wiki import setup_wiki
     load_config(app, dirpath=root)
     if debug is not None:
         app.config['DEBUG'] = debug
+    update_search_index()
     update_system_info()
     if browser:
         from threading import Timer
@@ -32,7 +33,7 @@ def main():
     import argparse
 
     parser = argparse.ArgumentParser(
-        description='NEOrg - Numerical Simulation Organizer')
+        description='NEOrg - Numerical Experiment Organizer')
     subparsers = parser.add_subparsers()
 
     # init
