@@ -14,10 +14,11 @@ import os
 import re
 import tempfile
 import shutil
+import urllib
 from nose.tools import raises, assert_raises
 
 from neorg import web
-from neorg.config import DefaultConfig
+from neorg.config import DefaultConfig, set_config
 from neorg.tests.utils import trim, CaptureStdIO, ChangeNEOrgVersion
 
 TMP_PREFIX = 'neorg-tmp'
@@ -45,19 +46,26 @@ def urljoin(a, *p):
 def setup_app():
     web.app.config.from_object(DefaultConfig)
     (db_fd, web.app.config['DATABASE']) = tempfile.mkstemp(prefix=TMP_PREFIX)
-    web.app.config['DATADIRPATH'] = tempfile.mkdtemp(prefix=TMP_PREFIX)
+    dirpath = tempfile.mkdtemp(prefix=TMP_PREFIX)  # = NEORG_ROOT
+    set_config(web.app.config, dirpath)
+    os.mkdir(web.app.config['NEORG_DIR'])
     web.app.config['SECRET_KEY'] = 'key for testing'
     app = web.app.test_client()
     web.init_db()
+    web.update_search_index()
 
     from neorg.wiki import setup_wiki, gene_html
     setup_wiki()
-    return (app, db_fd, gene_html)
+
+    def gene_html_with_encode(*args, **kwds):
+        return gene_html(*args, **kwds).encode('utf-8')
+
+    return (app, db_fd, gene_html_with_encode)
 
 
 def teardown_app():
     os.remove(web.app.config['DATABASE'])
-    shutil.rmtree(web.app.config['DATADIRPATH'])
+    shutil.rmtree(web.app.config['NEORG_ROOT'])
 
 
 class TestNEOrgWebSlow(object):
@@ -82,8 +90,8 @@ class TestNEOrgWebFast(object):
     def tearDownClass(cls):
         teardown_app()
 
-
-if os.environ.get("NEORG_FASTTEST") or False:
+NEORG_FASTTEST = os.environ.get("NEORG_FASTTEST") or False
+if NEORG_FASTTEST:
     TestNEOrgWebBase = TestNEOrgWebFast
 else:
     TestNEOrgWebBase = TestNEOrgWebSlow
@@ -92,49 +100,6 @@ else:
 class TestNEOrgWeb(TestNEOrgWebBase):
     num_test = 3  # Number of test for generated page
 
-    def test_system_info(self):
-       from neorg import __version__
-       from neorg.web import system_info
-       sysinfo = system_info()
-       assert sysinfo['version'] == __version__
-       assert sysinfo['updated'].startswith('20')
-       assert sysinfo['updated'][:4].isdigit()
-
-    def test_update_system_info_to_current(self):
-        from neorg.web import update_system_info, system_info
-        sysinfo_0 = system_info()
-        update_system_info()
-        sysinfo_1 = system_info()
-        assert sysinfo_0 == sysinfo_1  # nothing changed
-
-    def test_update_system_info_to_newer(self):
-        import neorg
-        from neorg.web import update_system_info
-        newer_version = '1.0.0'
-        assert newer_version > neorg.__version__  # make sure it is newer
-        with ChangeNEOrgVersion(newer_version):
-            # version has been changed
-            assert_raises(AssertionError, self.test_system_info)
-            # update success w/o an error
-            update_system_info()
-            # check if the version was updated
-            self.test_system_info()
-        # DB has been changed
-        assert_raises(AssertionError, self.test_system_info)
-
-    def test_update_system_info_to_older(self):
-        import neorg
-        from neorg.web import update_system_info
-        older_version = '0.0.0'
-        assert older_version < neorg.__version__  # make sure it is older
-        with ChangeNEOrgVersion(older_version):
-            # version has been changed
-            assert_raises(AssertionError, self.test_system_info)
-            # update fails wtih an RuntimeError
-            assert_raises(RuntimeError, update_system_info)
-        # DB has not been changed
-        self.test_system_info()
-
     @staticmethod
     def gene_page_paths(num, prefix="test"):
         yield ""
@@ -182,9 +147,9 @@ class TestNEOrgWeb(TestNEOrgWebBase):
         delete_path = urljoin('/', page_path, "_delete")
         response = self.app.post(delete_path, data={
             'yes': 'Yes',
-            }, follow_redirects=True)
+            })
         assert page_html not in response.data
-        assert '<form action="/_save"' in response.data
+        assert response.location == "http://localhost/"
 
     def check_delete_no(self, page_path, page_text):
         (save_response, page_html,
@@ -337,6 +302,12 @@ class TestNEOrgWeb(TestNEOrgWebBase):
         assert "path: %s" % page_path in response.data
         assert "relpath: /%s" % '/'.join(args) in response.data
 
+        # clean up is needed to avoid the template page to match
+        # another page. this is required when NEORG_FASTTEST is
+        # specified.
+        if NEORG_FASTTEST:
+            self.check_delete_yes(temp_path, '')
+
     def test_temp(self):
         page_root = 'TestTempRoot'
         num_max = 3
@@ -374,9 +345,119 @@ class TestNEOrgWeb(TestNEOrgWebBase):
                 in stderr)
         assert '<h1>Failed to generate HTML</h1>' in page_html
 
+    def get_search_response(self, query):
+        quoted = urllib.quote_plus(query)
+        return self.app.get('/_search?q={0}'.format(quoted))
+
+    @staticmethod
+    def assert_page_path_in_search_result(page_path, response):
+        page_path_link = '<h2><a href="/{0}">'.format(page_path)
+        assert page_path_link in response.data, \
+               "page_path '{0}' should be in the response"
+
+    def check_search_match(self, page_path, page_text, query):
+        self.check_save(page_path, page_text)
+        response = self.get_search_response(query)
+        self.assert_page_path_in_search_result(page_path, response)
+
+    @raises(AssertionError)
+    def check_search_no_match(self, page_path, page_text, query):
+        self.check_search_match(page_path, page_text, query)
+
+    def test_search(self):
+        num_test = 3
+        for ((page_path, page_text), query) in zip(
+            self.gene_pages('TestSearchMatch', num_test),
+            map('match{0} query{0}'.format, range(num_test))
+            ):
+            page_text = ' '.join([page_text, query])
+            yield (self.check_search_match,
+                   page_path, page_text, query)
+
+        for ((page_path, page_text), query) in zip(
+            self.gene_pages('TestSearchNoMatch', num_test),
+            map('match{0} query{0}'.format, range(num_test))
+            ):
+            yield (self.check_search_no_match,
+                   page_path, page_text, query)
+
+    def test_search_deleted(self):
+        query = 'some_long_string_to_test_search_deleted.'
+        page_path = 'TestSearchDeleted'
+        page_text = trim("""
+        This page will be deleted
+        -------------------------
+
+        {0}.
+        """.format(query))
+        self.check_search_match(page_path, page_text, query)
+        self.check_delete_yes(page_path, page_text)  # delete
+        response = self.get_search_response(query)
+        assert_raises(AssertionError,
+                      self.assert_page_path_in_search_result,
+                      page_path, response)  #no match
+
+    def test_jump_to_descendants(self):
+        page_path = 'TestJumpToDesc/SubPage'
+        self.check_save(page_path, 'subpage exists')
+
+        # if sub-page(s) exists, show them
+        response = self.app.get('TestJumpToDesc/')
+        assert (response.location ==
+                "http://localhost/TestJumpToDesc/_descendants")
+
+        # if no sub-page exists, jump to the edit form
+        response = self.app.get('TestJumpToDesc/NoSubPage/')
+        assert (response.location ==
+                "http://localhost/TestJumpToDesc/NoSubPage/_edit")
+
+
 
 class TestNEOrgWebWithEmptyDB(TestNEOrgWebSlow):
 
+    def test_system_info(self):
+        from neorg import __version__
+        from neorg.web import system_info
+        sysinfo = system_info()
+        assert sysinfo['version'] == __version__
+        assert sysinfo['updated'].startswith('20')
+        assert sysinfo['updated'][:4].isdigit()
+
+    def test_update_system_info_to_current(self):
+        from neorg.web import update_system_info, system_info
+        sysinfo_0 = system_info()
+        update_system_info()
+        sysinfo_1 = system_info()
+        assert sysinfo_0 == sysinfo_1  # nothing changed
+
+    def test_update_system_info_to_newer(self):
+        import neorg
+        from neorg.web import update_system_info
+        newer_version = '1.0.0'
+        assert newer_version > neorg.__version__  # make sure it is newer
+        with ChangeNEOrgVersion(newer_version):
+            # version has been changed
+            assert_raises(AssertionError, self.test_system_info)
+            # update success w/o an error
+            update_system_info()
+            # check if the version was updated
+            self.test_system_info()
+        # DB has been changed
+        assert_raises(AssertionError, self.test_system_info)
+
+    def test_update_system_info_to_older(self):
+        import neorg
+        from neorg.web import update_system_info
+        older_version = '0.0.0'
+        assert older_version < neorg.__version__  # make sure it is older
+        with ChangeNEOrgVersion(older_version):
+            # version has been changed
+            assert_raises(AssertionError, self.test_system_info)
+            # update fails wtih an RuntimeError
+            assert_raises(RuntimeError, update_system_info)
+        # DB has not been changed
+        self.test_system_info()
+
     def test_empty_db(self):
         response = self.app.get('/')
         assert response.location == "http://localhost/_edit"
