@@ -1,3 +1,4 @@
+# -*- coding: utf-8 -*-
 from __future__ import with_statement
 import os
 import re
@@ -8,6 +9,7 @@ from flask import (Flask, request, g, redirect, url_for,
 import jinja2
 from neorg.config import DefaultConfig
 from neorg.wiki import gene_html, safecall
+from neorg import search
 
 
 def regex_from_temp_path(path):
@@ -128,7 +130,16 @@ def find_descendants(path):
     return filter_descendants(path, path_list)
 
 
-ROOT_TITLE = 'Organize your experiments and find out more!'
+def has_descendants(path):
+    return bool(find_descendants(path))
+
+
+def path_as_title(path):
+    """
+    Convert page path to a title
+    """
+    return u' « '.join(reversed(path.split('/')))
+
 
 app = Flask('neorg')
 app.config.from_object(DefaultConfig)
@@ -205,6 +216,28 @@ def update_system_info():
             .format(oldver, curver))
 
 
+def get_search_index():
+    return search.get_index(app.config['SEARCHINDEX'])
+
+
+def update_search_index():
+    """
+    Update all search index or create new index.
+
+    .. warning::
+
+       Do NOT use this in app.
+       Call this function once just before `app.run`.
+
+    """
+    with closing(connect_db()) as db:
+        pages = db.execute('select * from pages').fetchall()
+    doc_list = [dict(
+        zip(('page_path', 'page_text'), doc)) for doc in pages]
+    ix = get_search_index()  # make new index if it does not exist
+    search.update_all(ix, doc_list)
+
+
 @app.before_request
 def before_request():
     """Make sure we are connected to the database each request."""
@@ -250,6 +283,7 @@ def delete(page_path):
             'values (?, ?, 0)',
             [page_path, ''])
         g.db.commit()
+        search.delete(get_search_index(), page_path)
         flash('Page "%s" was deleted.' % page_path)
         return redirect(url_for('page', page_path=''))
     elif request.form.get('no') == 'No':
@@ -262,7 +296,7 @@ def delete(page_path):
 def confirm_delete(page_path):
     (page_text, page_html) = get_page_text_and_html(page_path)
     return render_template("confirm_delete.html",
-                           title='Delete this page - ' + (page_path or ROOT_TITLE),
+                           title=path_as_title(page_path),
                            page_path=page_path,
                            page_html=page_html)
 
@@ -271,16 +305,18 @@ def confirm_delete(page_path):
 @app.route('/<path:page_path>/_save', methods=['POST'])
 def save(page_path):
     if request.form.get('save') == 'Save':
-        if get_page_text(page_path) == request.form['page_text']:
+        page_text = request.form['page_text']
+        if get_page_text(page_path) == page_text:
             flash('No change was found.')
             return redirect(url_for("page", page_path=page_path))
         g.db.execute(
             'insert or replace into pages (page_path, page_text) values (?, ?)',
-            [page_path, request.form['page_text']])
+            [page_path, page_text])
         g.db.execute(
             'insert into page_history (page_path, page_text) values (?, ?)',
-            [page_path, request.form['page_text']])
+            [page_path, page_text])
         g.db.commit()
+        search.update(get_search_index(), page_path, page_text)
         flash('Saved!')
         return redirect(url_for("page", page_path=page_path))
     elif request.form.get('preview') == 'Preview':
@@ -292,8 +328,8 @@ def save(page_path):
         else:
             flash('Previewing... Not yet saved!')
         return render_template(
-            "edit.html",
-            title='Preview - ' + (page_path or ROOT_TITLE),
+            "preview.html",
+            title=path_as_title(page_path),
             page_path=page_path,
             page_text=page_text,
             page_html=page_html)
@@ -309,7 +345,7 @@ def save(page_path):
 def edit(page_path):
     (page_text, page_html) = get_page_text_and_html(page_path)
     return render_template("edit.html",
-                           title='Edit - ' + (page_path or ROOT_TITLE),
+                           title=path_as_title(page_path),
                            page_path=page_path,
                            page_html=page_html,
                            page_text=page_text if page_text else '')
@@ -321,7 +357,7 @@ def page(page_path):
     (page_text, page_html) = get_page_text_and_html(page_path)
     if page_text:
         return render_template("page.html",
-                               title=page_path or ROOT_TITLE,
+                               title=path_as_title(page_path),
                                page_path=page_path,
                                page_html=page_html)
     else:
@@ -329,7 +365,13 @@ def page(page_path):
         if generated:
             return generated
         else:
-            return redirect(url_for('edit', page_path=page_path))
+            if has_descendants(page_path):
+                flash('Page {0} does not exist. Showing sub-pages.'
+                      .format(page_path))
+                return redirect(url_for('descendants',
+                                        page_path=page_path))
+            else:
+                return redirect(url_for('edit', page_path=page_path))
 
 
 _HTML_TEMP_GENE_TEXT_FAIL = """
@@ -367,7 +409,7 @@ def gene_from_template(page_path):
             page_html = gene_html(page_text, page_path,
                                   _debug=app.config['DEBUG'])
         return render_template("page.html",
-                               title=page_path or ROOT_TITLE,
+                               title=path_as_title(page_path),
                                temp_path=temp_path,
                                page_path=page_path,
                                page_html=page_html)
@@ -382,7 +424,7 @@ def list_descendants(page_path):
 def descendants(page_path):
     link_list = list_descendants(page_path)
     return render_template("descendants.html",
-                           title='Descendants - ' + (page_path or ROOT_TITLE),
+                           title=path_as_title(page_path),
                            link_list=link_list,
                            page_path=page_path)
 
@@ -398,7 +440,7 @@ def history(page_path):
     page_history = [dict(zip(page_history_keys, row))
                     for row in reversed(page_history_list)]
     return render_template("history.html",
-                           title='History - ' + (page_path or ROOT_TITLE),
+                           title=path_as_title(page_path),
                            page_path=page_path,
                            page_history=page_history)
 
@@ -412,7 +454,7 @@ def old(page_path, history_id):
     page_html = gene_html(page_text[0], page_path,
                           _debug=app.config['DEBUG'])
     return render_template("page.html",
-                           title=page_path or ROOT_TITLE,
+                           title=path_as_title(page_path),
                            page_path=page_path,
                            page_html=page_html)
 
@@ -442,3 +484,16 @@ def help(filename):
     else:
         return redirect(url_for('static',
                                 filename=os.path.join('help', filename)))
+
+
+@app.route('/_search')
+def search_results():
+    query = request.args.get('q')
+    if query:
+        results = search.search(get_search_index(), query, get_page_text)
+        return render_template("search.html",
+                               title=query,
+                               search_query=query,
+                               results=list(results))
+    else:
+        return redirect(url_for('page', page_path=''))
